package fr.smallcrew.rest.converter;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import java.util.Map;
import java.util.Set;

public class DataObjectResolver implements InitializingBean, ApplicationContextAware {
  private ApplicationContext applicationContext;
  private final Set<String> packagesToScan;
  private final Set<String> classesToScan;

  public DataObjectResolver(Set<String> packagesToScan, Set<String> classesToScan) {
    this.packagesToScan = packagesToScan;
    this.classesToScan = classesToScan;
  }

  @Override
  public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
    this.applicationContext = applicationContext;
  }

  @Override
  public void afterPropertiesSet() throws Exception {
    Map<String, RestConverter> converters = applicationContext.getBeansOfType(RestConverter.class);

    for (RestConverter converter : converters.values()) {
      converter.addPackage(packagesToScan);
      converter.addClasses(classesToScan);
      converter.consolidate();
    }
  }
}
