package fr.smallcrew.rest.data;

import org.springframework.data.domain.Page;

import javax.xml.bind.annotation.XmlRootElement;
import java.util.Collection;

@XmlRootElement
public class WebPage<T> {
  private int number;
  private int size;
  private int totalPages;
  private int numberOfElements;
  private long totalElements;
  private boolean hasPreviousPage;
  private boolean isFirstPage;
  private boolean hasNextPage;
  private boolean isLastPage;
  private boolean hasContent;
  private WebSort sort;
  private Collection<T> content;

  public WebPage() {
  }

  private WebPage(Page<T> page) {
    number = page.getNumber();
    size = page.getSize();
    totalPages = page.getTotalPages();
    numberOfElements = page.getNumberOfElements();
    totalElements = page.getTotalElements();
    hasPreviousPage = page.hasPreviousPage();
    isFirstPage = page.isFirstPage();
    hasNextPage = page.hasNextPage();
    isLastPage = page.isLastPage();
    content = page.getContent();
    hasContent = page.hasContent();
    sort = new WebSort(page.getSort());
  }

  public static <T> WebPage<T> from(Page<T> page) {
    return new WebPage<T>(page);
  }

  public int getNumber() {
    return number;
  }

  public void setNumber(int number) {
    this.number = number;
  }

  public int getSize() {
    return size;
  }

  public void setSize(int size) {
    this.size = size;
  }

  public int getTotalPages() {
    return totalPages;
  }

  public void setTotalPages(int totalPages) {
    this.totalPages = totalPages;
  }

  public int getNumberOfElements() {
    return numberOfElements;
  }

  public void setNumberOfElements(int numberOfElements) {
    this.numberOfElements = numberOfElements;
  }

  public long getTotalElements() {
    return totalElements;
  }

  public void setTotalElements(long totalElements) {
    this.totalElements = totalElements;
  }

  public boolean getHasPreviousPage() {
    return hasPreviousPage;
  }

  public void setHasPreviousPage(boolean hasPreviousPage) {
    this.hasPreviousPage = hasPreviousPage;
  }

  public boolean isFirstPage() {
    return isFirstPage;
  }

  public void setFirstPage(boolean firstPage) {
    isFirstPage = firstPage;
  }

  public boolean getHasNextPage() {
    return hasNextPage;
  }

  public void setHasNextPage(boolean hasNextPage) {
    this.hasNextPage = hasNextPage;
  }

  public boolean isLastPage() {
    return isLastPage;
  }

  public void setLastPage(boolean lastPage) {
    isLastPage = lastPage;
  }

  public boolean getHasContent() {
    return hasContent;
  }

  public void setHasContent(boolean hasContent) {
    this.hasContent = hasContent;
  }

  public WebSort getSort() {
    return sort;
  }

  public void setSort(WebSort sort) {
    this.sort = sort;
  }

  public Collection<T> getContent() {
    return content;
  }

  public void setContent(Collection<T> content) {
    this.content = content;
  }
}
