package fr.smallcrew.rest.view;

import com.google.common.base.Optional;
import fr.smallcrew.rest.converter.RestConverter;
import fr.smallcrew.rest.data.Response;
import fr.smallcrew.rest.data.wadl.Application;
import fr.smallcrew.rest.exception.ResourceExceptionHandler;
import fr.smallcrew.rest.servlet.filter.ExecutionTimeFilter;
import org.springframework.http.MediaType;
import org.springframework.http.server.ServletServerHttpResponse;
import org.springframework.validation.BindingResult;
import org.springframework.web.servlet.view.AbstractView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;
import java.util.Set;

public class RestView extends AbstractView {
  private final RestConverter converter;
  private final Set<Class<?>> filterClassType;
  private final MediaType mediaType;

  public RestView(RestConverter converter, Set<Class<?>> filterClassType) throws Exception {
    super();
    this.converter = converter;
    this.filterClassType = filterClassType;
    this.mediaType = converter.getSupportedMediaTypes().get(0);
    setContentType(mediaType.toString());
    setExposePathVariables(false);
    filterClassType.add(Application.class);
  }

  @Override
  protected void renderMergedOutputModel(Map<String, Object> data, HttpServletRequest request, HttpServletResponse response) throws Exception {
    converter.write(response(request, data), mediaType, new ServletServerHttpResponse(response));
  }

  private Object response(HttpServletRequest request, Map<String, Object> data) {
    Object object = firstModel(data);
    return dataResponse(object).or(standardResponse(request, object));
  }

  private Optional<Object> dataResponse(Object data) {
    Optional<Object> optional = Optional.absent();
    if (data != null && filterClassType.contains(data.getClass())) {
      optional = Optional.of(data);
    }
    return optional;
  }

  private Response standardResponse(HttpServletRequest request, Object data) {
    return new Response(!exceptionHappened(request), executionTime(request), data);
  }

  private boolean exceptionHappened(HttpServletRequest request) {
    return request.getAttribute(ResourceExceptionHandler.EXCEPTION_HAPPENED) != null;
  }

  private Object firstModel(Map<String, Object> model) {
    for (Map.Entry<String, Object> entry : model.entrySet()) {
      if (!(entry.getValue() instanceof BindingResult)) {
        return entry.getValue();
      }
    }
    return null;
  }

  private Integer executionTime(HttpServletRequest request) {
    Long startTime = (Long) request.getAttribute(ExecutionTimeFilter.START_EXECUTION_TIME);
    if (startTime != null) {
      return (int) (System.currentTimeMillis() - startTime);
    }
    return null;
  }
}

