package fr.smallcrew.rest.view;

import com.google.common.base.Function;
import com.google.common.collect.Collections2;
import com.google.common.collect.ImmutableList;
import org.springframework.http.MediaType;
import org.springframework.web.servlet.View;
import org.springframework.web.servlet.view.ContentNegotiatingViewResolver;

import java.util.Collection;
import java.util.List;
import java.util.Locale;

import static com.google.common.base.Preconditions.checkArgument;

public class RestViewResolver extends ContentNegotiatingViewResolver {
  private final Collection<MediaType> mediaTypes;
  private final View defaultView;

  public RestViewResolver(List<View> defaultViews, MediaType defaultContentType) {
    checkArgument(defaultViews != null && defaultViews.size() > 0, "At least one default view should be set");
    super.setDefaultViews(defaultViews);
    super.setDefaultContentType(defaultContentType);
    defaultView = getDefaultView(defaultViews, defaultContentType);
    mediaTypes = ImmutableList.copyOf(Collections2.transform(defaultViews, new Function<View, MediaType>() {
      @Override
      public MediaType apply(View input) {
        return MediaType.valueOf(input.getContentType());
      }
    }));
  }

  @Override
  public View resolveViewName(String viewName, Locale locale) throws Exception {
    View view = super.resolveViewName(viewName, locale);
    if (view == null) {
      return defaultView;
    }
    return view;
  }

  private View getDefaultView(List<View> defaultViews, MediaType defaultContentType) {
    for (View view : defaultViews) {
      if (view.getContentType().equals(defaultContentType.toString())) {
        return view;
      }
    }
    return defaultViews.get(0);
  }

  public Collection<MediaType> getMediaTypes() {
    return mediaTypes;
  }
}
