package fr.smallcrew.rest.wadl;

import fr.smallcrew.rest.data.wadl.Application;
import org.springframework.http.MediaType;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerMapping;

import javax.servlet.http.HttpServletRequest;
import javax.xml.namespace.QName;
import java.util.Collection;

import static org.springframework.util.StringUtils.uncapitalize;

public class ApplicationBuilder {
  private final RequestMappingHandlerMapping handlerMapping;
  private final HttpServletRequest request;
  private final Collection<MediaType> mediaTypes;

  private ApplicationBuilder(HttpServletRequest request, RequestMappingHandlerMapping handlerMapping,
                             Collection<MediaType> mediaTypes) {
    this.handlerMapping = handlerMapping;
    this.request = request;
    this.mediaTypes = mediaTypes;
  }

  public static ApplicationBuilder from(HttpServletRequest request, RequestMappingHandlerMapping handlerMapping,
                                        Collection<MediaType> mediaTypes) {
    return new ApplicationBuilder(request, handlerMapping, mediaTypes);
  }

  public Application build() {
    Application application = new Application();
    application.getResources().add(ResourcesBuilder.from(request, handlerMapping, mediaTypes).build());
    return application;
  }

  public static QName qName(Class<?> parameterType) {
    return new QName("http://www.w3.org/2001/XMLSchema", uncapitalize(parameterType.getSimpleName()));
  }
}
