package fr.smallcrew.rest.wadl;

import com.google.common.base.Strings;
import fr.smallcrew.rest.data.wadl.Param;
import fr.smallcrew.rest.data.wadl.ParamStyle;
import fr.smallcrew.rest.data.wadl.Representation;
import fr.smallcrew.rest.data.wadl.Request;
import org.springframework.http.MediaType;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ValueConstants;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.Collection;

import static fr.smallcrew.rest.wadl.ApplicationBuilder.qName;

public class RequestBuilder {
  private final java.lang.reflect.Method method;
  private final Collection<MediaType> mediaTypes;

  public RequestBuilder(Method method, Collection<MediaType> mediaTypes) {
    this.method = method;
    this.mediaTypes = mediaTypes;
  }

  public static RequestBuilder from(Method method, Collection<MediaType> mediaTypes) {
    return new RequestBuilder(method, mediaTypes);
  }

  public Request build() {
    Annotation[][] annotationsArray = method.getParameterAnnotations();
    Class<?>[] parameterTypes = method.getParameterTypes();
    Request request = new Request();
    for (int i = 0; i < annotationsArray.length; i++) {
      for (Annotation annotation : annotationsArray[i]) {
        maybeAddParam(annotation, request, parameterTypes[i]);
      }
    }
    return request;
  }

  private void maybeAddParam(Annotation annotation, Request request, Class<?> parameterType) {
    if (annotation instanceof RequestParam) {
      addParam((RequestParam) annotation, request, parameterType);
    } else if (annotation instanceof PathVariable) {
      addParam((PathVariable) annotation, request, parameterType);
    } else if (annotation instanceof RequestBody) {
      addParam((RequestBody) annotation, request, parameterType);
    }
  }

  private void addParam(PathVariable annotation, Request request, Class<?> parameterType) {
    Param param = new Param();
    param.setStyle(ParamStyle.TEMPLATE);
    param.setName(annotation.value());
    param.setRequired(true);
    param.setType(qName(parameterType));
    request.getParam().add(param);
  }

  private void addParam(RequestParam annotation, Request request, Class<?> parameterType) {
    Param param = new Param();
    param.setStyle(ParamStyle.QUERY);
    if (!Strings.isNullOrEmpty(annotation.value())) {
      param.setName(annotation.value());
    }
    param.setName(annotation.value());
    if (!ValueConstants.DEFAULT_NONE.equals(annotation.defaultValue())) {
      param.setDefault(annotation.defaultValue());
    }
    param.setRequired(annotation.required());
    param.setType(qName(parameterType));
    request.getParam().add(param);
  }

  private void addParam(RequestBody annotation, Request request, Class<?> parameterType) {
    Param param = new Param();
    param.setName(StringUtils.uncapitalize(parameterType.getSimpleName()));
    param.setType(qName(parameterType));
    param.setRequired(true);

    for (MediaType mediaType : mediaTypes) {
      Representation representation = new Representation();
      representation.setMediaType(mediaType.toString());
      representation.getParam().add(param);
      request.getRepresentation().add(representation);
    }
  }
}
